// BeerExpert.java  for CISC474, H07, P. Conrad, Spring 2007

package edu.udel.cisc474.pconrad.BeerAdvisor.model;
import java.util.*;
import java.sql.*;   // Needed for doing database stuff

/**
 * <p>
 * BeerExpert provides recommendations for Beers based on
 * user input.   It provides an abstraction of the actual data
 * source, which can be hard coded values, or an SQL database.
 * <p>
 * BeerExpert represents the "Model" part of a beer advisory 
 * system based on the
 * Model-View-Controller design pattern.
 * <p>
 * The original is based on code from p. 82 of 
 * "Head First Servlets and JSP", 1st Edition.
 * 
 * @author    P.Conrad, based on code from HFSJ
 */


public class BeerExpert {

    /**
     * Create a BeerExpert object that gets its information from a MySQL
     * relational database, specified by the paramaters passed in to the constructor.
     *
     * @param jdbcURL  for example,
     *                 <code>"jdbc:mysql://cisc474.acad.cis.udel.edu/pconrad"
     *                 The <code>"jdbc.mysql://"</code> should be
     *                 exactly that.  The
     *                 <code>cisc474.acad.cis.udel.edu</code> part is
     *                 the host where the database is running, and the
     *                 <code>pconrad</code> part is the name of the
     *                 <em>database</em>, not the userid.  In this
     *                 case, those happen to be the same, but don't be
     *                 confused on this point.
     * @param username for example, <code>"pconrad"</code>---the MySQL
     *                 username, <em>not</em> your Unix username!
     * @param password for example, <code>"mySecret123"</code>---the
     *                 MySQL password, <em>not</em> your Unix
     *                 password!
     */

    private String jdbcURL_;
    private String username_;
    private String password_;
    private final String prefix_ = "BA_";

     /* Explanation of prefix_: Many hosting providers only allow a
        user one database.  So, if you want to have multiple
        applications running, it is helpful to prefix all of your
        table names with some prefix, e.g. "BA_" to distinguish them
        from tables for some other application.</p> <p>This is
        admiteddly a hack.  In a perfect world, you'd separate
        applications into different databases, but in the real world,
        sometimes this is not feasible. */

    public BeerExpert(String jdbcURL, String username, String password)
    {

	// initialize the private data members

	jdbcURL_ = jdbcURL;
	username_ = username;
	password_ = password;

    }


/**
 * Given a color of beer, returns a list of beers that
 * match that color. 
 * 
 * @param color       the color of beer the user prefers (e.g. <code>"amber"</code>)
 * @return            a List<String> object containing the names of the
 *                    beers that match that color.
 * @see BeerExpertTest
 */

    public List<String> getBrands(String color) {

	List<String> brands = new ArrayList<String>();

	// Create the connection to the cars db	
	
	Connection myCon;
	Statement myStmt;


	// Make sure we can load the MySQL driver 

	try {
	    Class.forName("com.mysql.jdbc.Driver").newInstance();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}


	try {
	    myCon = DriverManager.getConnection(jdbcURL_,username_,password_);
	}
	catch (SQLException e) {
	    e.printStackTrace();
	    return brands;
	}

	// Create a statement for SQL queries

	try {
	    myStmt = myCon.createStatement();
	}
	catch (Exception e){
	    e.printStackTrace();
	    return brands;
	}
	
	// Set up the query.  Use a ? where we will put in user input, and use
	// the "prepared statement" technique, in order to avoid SQL Injection attacks

	String tableName = prefix_ + "Beers";
	String query = "SELECT beer, color FROM " + tableName + "  WHERE color=?";
	
	// Turn our query into a prepared statement, and then stick the value of "color"
	// in where the first ? appears.   The setString method of prepared statement 
	
	ResultSet rs;
	
	try {
	    PreparedStatement prepStmt = myCon.prepareStatement(query);
	    prepStmt.setString(1, color);
	    rs = prepStmt.executeQuery();
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return brands;
	}

	// ** change next line to true to make JUnit tests fail
	final boolean stubForCheckingJUnitTests = false;

	if (stubForCheckingJUnitTests)
	    return brands; // returns an empty list

	try {
	    while (rs.next()) {
		String thisBeer = rs.getString("beer");
		String thisColor = rs.getString("color");
		
		brands.add(thisBeer);
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return brands;
	}	

	// finally, we can return the completed list of brands!

	return brands;

    }

    /**
       main is a command line "view/controller" that 
       takes a color as first command line arg, calls
       getBrands, and lists all returned brands
    */

    public static void main(String args[]) {

	// check command line arguments

	if (args.length != 4)
	    {
		System.err.println("Usage: java BeerExpert color jdbcURL mySQLusername mySQLpassword");
		System.exit(1);
	    }

	String color = args[0];
	String jdbcURL = args[1];
	String username = args[2];
	String password = args[3];

	BeerExpert be = new BeerExpert(jdbcURL,username,password);
	
	List<String> brands = be.getBrands(color);

	if (brands.size() == 0)
	    System.out.println("No beers found with color=" + color);
	else
	    {
		System.out.println("Try:");
		for (String brand: brands)
		    System.out.println(" " + brand);
	    }
	System.exit(0);
    }


}
